package net.sf.jguard.jsf.authentication.callbacks;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;

import java.util.Locale;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.LanguageCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindings;
import net.sf.jguard.core.authentication.callbacks.InetAddressCallback;
import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;
import net.sf.jguard.jee.authentication.callbacks.HttpServletCallbackHandler;
import net.sf.jguard.jsf.AccessListener;

public class JSFCallbackHandler extends HttpServletCallbackHandler {

	private static final Logger logger = LoggerFactory.getLogger(JSFCallbackHandler.class);
	
	public JSFCallbackHandler(AuthenticationBindings authBindings,AccessContext context,List<AuthenticationSchemeHandler> authenticationSchemeHandlers){
        super(authBindings,context,authenticationSchemeHandlers);   
    }

	
	/**
     * we fill callbacks not directly related to an AuthenticationSchemeHandler like
     * {@link InetAddressCallback} and {@link LanguageCallback}.
     * @param cbks
     * @throws javax.security.auth.callback.UnsupportedCallbackException
     */
    @Override
    protected void handleNonSchemeCallbacks(List<Callback> cbks) throws UnsupportedCallbackException {
        FacesContext faceContext = (FacesContext)context.getAttribute(AccessListener.FACES_CONTEXT);
        ExternalContext extContext = faceContext.getExternalContext();
        Object request = extContext.getRequest();
        HttpServletRequest httpRequest = null;
        PortletRequest portletRequest = null;
        if (HttpServletRequest.class.isAssignableFrom(request.getClass())){
            httpRequest = (HttpServletRequest)request;
        }else{
            portletRequest = (PortletRequest)request;
        }
           
        for (Callback cb : cbks) {
            if (cb instanceof InetAddressCallback) {
                if(httpRequest==null){
                    continue;
                }
                String remoteAddress = httpRequest.getRemoteAddr();
                String remoteHost = httpRequest.getRemoteHost();
                InetAddressCallback inetAddressCallback = (InetAddressCallback) cb;
                inetAddressCallback.setHostAdress(remoteAddress);

                //the server is not configured to return the hostName.
                if (remoteAddress.equals(remoteHost)) {
                    String resolvedHostName = remoteAddress;
                    try {
                        resolvedHostName = HttpServletCallbackHandler.reverseDns(remoteAddress);
                    } catch (UnknownHostException uhe) {
                        logger.warn(" host bound to address " + remoteAddress + "cannot be resolved", uhe);
                        throw new UnsupportedCallbackException(cb, uhe.getMessage());
                    } catch (IOException ex) {
                        logger.equals(ex.getMessage());
                        throw new UnsupportedCallbackException(cb, ex.getMessage());
                    }
                    inetAddressCallback.setHostName(resolvedHostName);
                }
            } else if (cb instanceof LanguageCallback) {
                LanguageCallback languageCallback = (LanguageCallback) cb;
                Locale locale = extContext.getRequestLocale();
                languageCallback.setLocale(locale);
            }
        }
    }
}
