/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/


package net.sf.jguard.jsf.authorization;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authorization.AuthorizationBindings;
import net.sf.jguard.core.filters.FilterChain;
import net.sf.jguard.jsf.ExternalContextUtil;
import net.sf.jguard.jsf.permissions.JSFPermissionFactory;

import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.application.NavigationHandler;
import javax.faces.context.FacesContext;
import java.security.Permission;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * JSF implementation of AuthorizationBindings.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class JSFAuthorizationBindings implements AuthorizationBindings {

    /**
     * forward to the "accessDenied" view the user and add to the FacesContext with
     * a WARN severity a FaceMessage.
     *
     * @param context
     */
    public void accessDenied(AccessContext context) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        String outcomeAccessDenied = "accessDenied";
        NavigationHandler nh = facesContext.getApplication().getNavigationHandler();
        nh.handleNavigation(facesContext, null, outcomeAccessDenied);
        String msg = getLocalizedAndFormattedMessage(facesContext, outcomeAccessDenied, null);
        facesContext.addMessage(outcomeAccessDenied, new FacesMessage(FacesMessage.SEVERITY_WARN, msg, msg));
    }

    public void doFilter(AccessContext context, FilterChain chain) {

    }

    public Permission getPermissionRequested(AccessContext context) {
        return new JSFPermissionFactory().getPermission(context);
    }

    public void setLastAccessDeniedPermission(AccessContext context,
                                              Permission permission) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        ExternalContextUtil.setAttribute(facesContext.getExternalContext(), CoreConstants.LAST_ACCESS_DENIED_PERMISSION, permission);
    }

    /**
     * add a FacesMessage with a "throwable" key, and an ERROR severity to the FacesContext.
     * it forwards also to a "throwable" outcome.
     *
     * @param accessContext
     * @param t
     */
    public void sendThrowable(AccessContext accessContext, Throwable t) {
        String outcomeThrowable = "throwable";
        FacesContext facesContext = FacesContext.getCurrentInstance();
        NavigationHandler nh = facesContext.getApplication().getNavigationHandler();
        nh.handleNavigation(facesContext, null, outcomeThrowable);
        String[] params = {t.getLocalizedMessage()};
        String msg = getLocalizedAndFormattedMessage(facesContext, outcomeThrowable, params);
        if ("".equals(msg)) {
            msg = t.getLocalizedMessage();
        }
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg);
        facesContext.addMessage(null, facesMsg);

    }

    /**
     * return an localized message according to the Locale from the view root,
     * localized with a message key and parameters
     *
     * @param context
     * @param messageKey
     * @param params     can be null
     * @return
     */
    private String getLocalizedAndFormattedMessage(FacesContext context, String messageKey, Object[] params) {
        // this method is inspired from this book excerpt:
        //http://www.onjava.com/pub/a/onjava/excerpt/JSF_chap8/index.html?page=3
        Application application = context.getApplication();
        String messageBundleName = application.getMessageBundle();
        Locale locale = context.getViewRoot().getLocale();
        if (messageBundleName == null || "".equals(messageBundleName)) {
            return "";
        }
        ResourceBundle rb = ResourceBundle.getBundle(messageBundleName, locale);
        String msgPattern = rb.getString(messageKey);
        String msg = msgPattern;
        if (params != null) {
            msg = MessageFormat.format(msgPattern, params);
        }
        return msg;
    }

}
